{% extends "account/base.html" %}

{% load url from future %}
{% load i18n %}

{% block head_title %}{% trans "Sign Up Closed" %}{% endblock %}

{% block content %}
<h1>{% trans "Sign Up Closed" %}</h1>

<p>{% trans "We are sorry, but the sign up is currently closed." %}</p>
{% endblock %}


